@@ -3,9 +3,12 @@ import sys
 import os
 
 import numpy as np
+import seaborn as sns
+import pandas as pd
 
 import math
 import matplotlib.colors as mcolors
+from matplotlib.lines import Line2D
 import scipy
 
 import matplotlib
@@ -165,7 +168,7 @@ class Visualize:
                 os.makedirs(self.save_dir)
         pass
     
-    def data(self, X, labels, title='Data', figsize=None, s=20, cmap='jet'):
+    def data(self, X, labels, title='Data', figsize=None, s=20, cmap='jet', azim=None, elev=None, colorbar=False):
         assert X.shape[1] <= 3, 'X.shape[1] must be either 2 or 3.'
         fig = plt.figure(figsize=figsize)
         if matplotlib.get_backend().startswith('Qt'):
@@ -174,15 +177,22 @@ class Visualize:
         if X.shape[1] == 2:
             plt.scatter(X[:,0], X[:,1], s=s, c=labels, cmap=cmap)
             plt.axis('image')
+            if colorbar:
+                plt.colorbar()
         elif X.shape[1] == 3:
             ax = fig.add_subplot(projection='3d')
-            ax.scatter(X[:,0], X[:,1], X[:,2], s=s, c=labels, cmap=cmap)
+            if azim and elev:
+                ax.view_init(azim=azim, elev=elev)
+            p = ax.scatter(X[:,0], X[:,1], X[:,2], s=s, c=labels, cmap=cmap)
             set_axes_equal(ax)
+            if colorbar:
+                fig.colorbar(p)
         plt.title(title)
+        
         plt.tight_layout()
         #plt.axis('off')
         if self.save_dir:
-            plt.savefig(self.save_dir+'/' + title + '.png') 
+            plt.savefig(self.save_dir+'/' + title + '.eps') 
         plt.show()
         
     def eigenvalues(self, lmbda, figsize=None):
@@ -217,7 +227,7 @@ class Visualize:
         if self.save_dir:
             if not os.path.isdir(self.save_dir+'/gamma'):
                 os.makedirs(self.save_dir+'/gamma')
-            plt.savefig(self.save_dir+'/gamma/'+str(i)+'.png') 
+            plt.savefig(self.save_dir+'/gamma/'+str(i)+'.eps') 
         plt.show()
     
     def eigenvector(self, X, phi, i, figsize=None, s=20):
@@ -239,7 +249,7 @@ class Visualize:
         if self.save_dir:
             if not os.path.isdir(self.save_dir+'/eigvecs'):
                 os.makedirs(self.save_dir+'/eigvecs')
-            plt.savefig(self.save_dir+'/eigvecs/'+str(i)+'.png') 
+            plt.savefig(self.save_dir+'/eigvecs/'+str(i)+'.eps') 
         plt.show()
     
     def grad_phi(self, X, phi, grad_phi, i, prop=0.01, figsize=None, s=20):
@@ -279,7 +289,7 @@ class Visualize:
         if self.save_dir:
             if not os.path.isdir(self.save_dir+'/grad_phi'):
                 os.makedirs(self.save_dir+'/grad_phi')
-            plt.savefig(self.save_dir+'/grad_phi/'+str(i)+'.png') 
+            plt.savefig(self.save_dir+'/grad_phi/'+str(i)+'.eps') 
         plt.show()
     
     def Atilde(self, X, phi, i, j, Atilde, figsize=None, s=20):
@@ -325,7 +335,7 @@ class Visualize:
         if self.save_dir:
             if not os.path.isdir(self.save_dir+'/Atilde'):
                 os.makedirs(self.save_dir+'/Atilde')
-            plt.savefig(self.save_dir+'/Atilde/'+str(i)+'_'+str(j)+'.png') 
+            plt.savefig(self.save_dir+'/Atilde/'+str(i)+'_'+str(j)+'.eps') 
         plt.show()
     
     def n_eigvecs_w_grad_lt(self, X, Atilde, thresh_prctile=None, figsize=(16,8), s=20):
@@ -401,7 +411,7 @@ class Visualize:
             if self.save_dir:
                 if not os.path.isdir(self.save_dir+'/n_eigvecs_w_grad_lt'):
                     os.makedirs(self.save_dir+'/n_eigvecs_w_grad_lt')
-                plt.savefig(self.save_dir+'/n_eigvecs_w_grad_lt/'+str(thresh)+'.png')
+                plt.savefig(self.save_dir+'/n_eigvecs_w_grad_lt/'+str(thresh)+'.eps')
             plt.show()
             
             if thresh_prctile is not None:
@@ -428,14 +438,85 @@ class Visualize:
             fig.colorbar(p)
         plt.title(title)
         if self.save_dir:
-            plt.savefig(self.save_dir+'/'+title+'.png') 
+            plt.savefig(self.save_dir+'/'+title+'.eps') 
         plt.show()
     
     def distortion_boxplot(self, zeta, title, figsize=None):
         fig = plt.figure(figsize=figsize)
         plt.boxplot([zeta],labels=[title], notch=True, patch_artist=True)
         if self.save_dir:
-            plt.savefig(self.save_dir+'/box_'+title+'.png') 
+            plt.savefig(self.save_dir+'/box_'+title+'.eps') 
+        plt.show()
+        
+    def global_distortion_viloinplot_overlay(self, dist_dicts, ylabel='$\log(\mathcal{G}_k)$',
+                                     title='violinplot for $\log(\mathcal{G}_k)$',
+                                     log_scale=True, figsize=None,
+                                     color=None, widths=0.5, lw=3,
+                                     offset1=1, offset2=0.01,
+                                     legend=True, rotation=0, filled=False):
+        # create figure and axes
+        fig, ax = plt.subplots(figsize=figsize)
+
+        xticks = []
+        xticklabels = []
+        x0 = 0
+        for ex_name,dist_dict in dist_dicts.items():
+            xticks.append(x0)
+            xticklabels.append(ex_name)
+            for k in dist_dict.keys():
+                if log_scale:
+                    data=pd.DataFrame(dist_dict[k]).applymap(lambda x: np.log(x))
+                else:
+                    data=pd.DataFrame(dist_dict[k])
+                parts = ax.violinplot(data, [x0], showmeans=False,
+                                       showmedians=False,
+                                       showextrema=False,
+                                       widths=widths)
+                x0 += offset2
+                for pc in parts['bodies']:
+                    pc.set_linewidth(lw)
+                    pc.set_alpha(1)
+                    if color is not None:
+                        pc.set_edgecolor(color[k])
+                    if not filled:
+                        pc.set_facecolor('none')
+                    else:
+                        pc.set_facecolor(color[k])
+                    pc.set_label(k)
+            x0 += offset1
+
+        if legend:
+            custom_lines = [
+                Line2D([0], [0], color=color[k], lw=lw, alpha=1) 
+                for k in color.keys()
+            ]
+            ax.legend(
+                custom_lines, 
+                [k for k in color.keys()],
+            )
+        
+        ax.set_xticks(xticks)
+        ax.set_xticklabels(xticklabels, rotation=rotation)
+        plt.ylabel(ylabel)
+        plt.title(title)
+        if self.save_dir:
+            plt.savefig(self.save_dir+'/global_distortion_violinplot_overlay.eps', format='eps') 
+        plt.show()
+        
+    def global_distortion_viloinplot(self, dist_dict, ylabel='$\log(D_k)$',
+                                     title='violinplot for $\log(D_k)$',
+                                     log_scale=True, figsize=None):
+        fig = plt.figure(figsize=figsize)
+        if log_scale:
+            sns.violinplot(data=pd.DataFrame(dist_dict).applymap(lambda x: np.log(x)))
+            plt.ylabel('$\log(D_k)$')
+            plt.title('Violinplot for $\log(D_k)$')
+        else:
+            sns.violinplot(data=pd.DataFrame(dist_dict))
+            plt.ylabel('$D_k$')
+            plt.title('Violinplot for $D_k$')
+        if self.save_dir:
+            plt.savefig(self.save_dir+'/global_distortion_violinplot.eps', format='eps') 
         plt.show()
     
     def dX(self, X, ddX, title, figsize=None, s=20):
@@ -465,7 +546,7 @@ class Visualize:
             fig.colorbar(p, ax=ax)
             ax.set_title('dX') 
         if self.save_dir:
-            plt.savefig(self.save_dir+'/'+title+'.png') 
+            plt.savefig(self.save_dir+'/'+title+'.eps') 
         plt.show()
     
     def intrinsic_dim(self, X, chi, figsize=None, s=20):
@@ -700,7 +781,7 @@ class Visualize:
             if self.save_dir:
                 if not os.path.isdir(self.save_dir+'/local_views/'+save_subdir):
                     os.makedirs(self.save_dir+'/local_views/'+save_subdir)
-                plt.savefig(self.save_dir+'/local_views/'+save_subdir+'/'+str(k)+'.png')
+                plt.savefig(self.save_dir+'/local_views/'+save_subdir+'/'+str(k)+'.eps')
             plt.show()
             
             first_plot = False
@@ -823,7 +904,7 @@ class Visualize:
             if self.save_dir:
                 if not os.path.isdir(self.save_dir+'/local_views/'+save_subdir):
                     os.makedirs(self.save_dir+'/local_views/'+save_subdir)
-                plt.savefig(self.save_dir+'/local_views/'+save_subdir+'/'+str(k)+'.png')
+                plt.savefig(self.save_dir+'/local_views/'+save_subdir+'/'+str(k)+'.eps')
             plt.show()
             
             first_plot = False
@@ -973,7 +1054,7 @@ class Visualize:
             if self.save_dir:
                 if not os.path.isdir(self.save_dir+'/intermediate_views/'+save_subdir):
                     os.makedirs(self.save_dir+'/intermediate_views/'+save_subdir)
-                plt.savefig(self.save_dir+'/intermediate_views/'+save_subdir+'/'+str(m)+'.png') 
+                plt.savefig(self.save_dir+'/intermediate_views/'+save_subdir+'/'+str(m)+'.eps') 
             plt.show()
             
             first_plot = False
@@ -1087,7 +1168,7 @@ class Visualize:
             if self.save_dir:
                 if not os.path.isdir(self.save_dir+'/local_high_low_distortion/'):
                     os.makedirs(self.save_dir+'/local_high_low_distortion/')
-                plt.savefig(self.save_dir+'/local_high_low_distortion/thresh='+str(thresh)+'.png') 
+                plt.savefig(self.save_dir+'/local_high_low_distortion/thresh='+str(thresh)+'.eps') 
             plt.show()
             
         
@@ -1205,7 +1286,7 @@ class Visualize:
             if self.save_dir:
                 if not os.path.isdir(self.save_dir+'/intermediate_high_low_distortion'):
                     os.makedirs(self.save_dir+'/intermediate_high_low_distortion')
-                plt.savefig(self.save_dir+'/intermediate_high_low_distortion/thresh='+str(thresh)+'.png') 
+                plt.savefig(self.save_dir+'/intermediate_high_low_distortion/thresh='+str(thresh)+'.eps') 
             plt.show()
     
     def seq_of_intermediate_views(self, X, c, seq, rho, Utilde, figsize=None, s=20, cmap='jet'):
@@ -1318,7 +1399,7 @@ class Visualize:
         if self.save_dir:
             if not os.path.isdir(self.save_dir+'/ge'):
                 os.makedirs(self.save_dir+'/ge')
-            plt.savefig(self.save_dir+'/ge/'+str(title)+'.png', bbox_inches = 'tight',pad_inches = 0)
+            plt.savefig(self.save_dir+'/ge/'+str(title)+'.eps', bbox_inches = 'tight',pad_inches = 0)
         plt.show()
     
     def global_embedding_images(self, X, img_shape, y, labels, cmap0, color_of_pts_on_tear=None, cmap1=None,
@@ -1421,7 +1502,7 @@ class Visualize:
             if self.save_dir:
                 if not os.path.isdir(self.save_dir+'/ge_img'):
                     os.makedirs(self.save_dir+'/ge_img')
-                plt.savefig(self.save_dir+'/ge_img/'+str(title)+'.png')
+                plt.savefig(self.save_dir+'/ge_img/'+str(title)+'.eps')
             plt.show()
     
     def global_embedding_images_v2(self, X, img_shape, y, labels, cmap0, color_of_pts_on_tear=None, cmap1=None,
@@ -1550,7 +1631,7 @@ class Visualize:
         if self.save_dir:
             if not os.path.isdir(self.save_dir+'/ge_img_v2'):
                 os.makedirs(self.save_dir+'/ge_img_v2')
-            plt.savefig(self.save_dir+'/ge_img_v2/'+str(title)+'.png')
+            plt.savefig(self.save_dir+'/ge_img_v2/'+str(title)+'.eps')
     
     def visualize_epoch_data(self, X, Ls):
         a=np.array(X)
