@@ -405,6 +405,9 @@ def visualize(fpath, threshs=[5,10,15], n_views=4, figsize1=(8,8),
                                     ldle.vis_opts['cmap_boundary'], 'images',
                                     offset_ratio=0.2, zoom=0.4, nx=8, ny=10, v_ratio=0.65, w_ratio=0.005,
                                     figsize=(10,12), s=80, to_remove=False, k_to_avoid=[], to_T=True)
+        ldle.vis.global_embedding(ldle.GlobalViews.y_final, X[:,0], 'summer',
+                                  ldle.GlobalViews.color_of_pts_on_tear_final, ldle.vis_opts['cmap_boundary'],
+                                  'Final Embedding 2', figsize=(5,5), s=50)
     
     if 'epoch' in fpath:
         Ls = all_data[4]
